@@ -1,4 +1,4 @@
-/* $Id: pjsua_app_cli.c 5170 2015-08-25 08:45:46Z nanang $ */
+/* $Id: pjsua_app_cli.c 5542 2017-01-23 06:15:14Z ming $ */
 /*
  * Copyright (C) 2008-2011 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2003-2008 Benny Prijono <benny@prijono.org>
@@ -151,6 +151,10 @@ static pj_cli_t		   *cli = NULL;
 static pj_cli_sess	   *cli_cons_sess = NULL;
 static pj_cli_front_end	   *telnet_front_end = NULL;
 
+#ifdef USE_GUI
+void displayLog(const char *msg, int len);
+#endif
+
 /** Forward declaration **/
 pj_status_t cli_setup_command(pj_cli_t *cli);
 void cli_destroy();
@@ -170,6 +174,9 @@ static void cli_log_writer(int level, const char *buffer, int len)
 {
     if (cli)
         pj_cli_write_log(cli, level, buffer, len);
+#ifdef USE_GUI
+    displayLog(buffer, len);
+#endif
 }
 
 pj_status_t cli_init()
@@ -207,10 +214,6 @@ pj_status_t cli_init()
     /* Init telnet frontend */
     if (app_config.cli_cfg.cli_fe & CLI_FE_TELNET) {
 	pj_cli_telnet_cfg *fe_cfg = &app_config.cli_cfg.telnet_cfg;
-	pj_pool_t *pool;
-
-	pool = pj_pool_create(cfg->pf, "cli_cp", 128, 128, NULL);
-	pj_assert(pool);
 
 	status = pj_cli_telnet_create(cli, fe_cfg, &telnet_front_end);
 	if (status != PJ_SUCCESS)
@@ -1088,7 +1091,7 @@ static pj_status_t cmd_set_presence_text(pj_cli_cmd_val *cval)
 	AVAILABLE, BUSY, OTP, IDLE, AWAY, BRB, OFFLINE, OPT_MAX
     };
 
-    choice = pj_strtol(&cval->argv[1]) - 1;
+    choice = (int)pj_strtol(&cval->argv[1]) - 1;
 
     pj_bzero(&elem, sizeof(elem));
     elem.type = PJRPID_ELEMENT_TYPE_PERSON;
@@ -1247,11 +1250,11 @@ static pj_status_t cmd_media_connect(pj_cli_cmd_val *cval, pj_bool_t connect)
     pj_status_t status;
 
     if (connect)
-	status = pjsua_conf_connect(pj_strtol(&cval->argv[1]),
-				    pj_strtol(&cval->argv[2]));
+	status = pjsua_conf_connect((int)pj_strtol(&cval->argv[1]),
+				    (int)pj_strtol(&cval->argv[2]));
     else
-	status = pjsua_conf_disconnect(pj_strtol(&cval->argv[1]),
-				       pj_strtol(&cval->argv[2]));
+	status = pjsua_conf_disconnect((int)pj_strtol(&cval->argv[1]),
+				       (int)pj_strtol(&cval->argv[2]));
 
     if (status == PJ_SUCCESS) {
 	static const pj_str_t success_msg = {"Success\n", 9};
@@ -1305,7 +1308,7 @@ static pj_status_t cmd_set_codec_prio(pj_cli_cmd_val *cval)
     int new_prio;
     pj_status_t status;
 
-    new_prio = pj_strtol(&cval->argv[2]);
+    new_prio = (int)pj_strtol(&cval->argv[2]);
     if (new_prio < 0)
 	new_prio = 0;
     else if (new_prio > PJMEDIA_CODEC_PRIO_HIGHEST)
@@ -1516,7 +1519,7 @@ static pj_status_t cmd_make_multi_call(pj_cli_cmd_val *cval)
 		     "(You currently have %d calls)\n",
 		     pjsua_call_get_count());
 
-    count = pj_strtol(&cval->argv[1]);
+    count = (int)pj_strtol(&cval->argv[1]);
     if (count < 1)
 	return PJ_SUCCESS;
 
@@ -1576,7 +1579,7 @@ static pj_status_t cmd_answer_call(pj_cli_cmd_val *cval)
 	pj_str_t hvalue;
 	pjsip_generic_string_hdr hcontact;
 
-	st_code = pj_strtol(&cval->argv[1]);
+	st_code = (int)pj_strtol(&cval->argv[1]);
 	if ((st_code < 100) || (st_code > 699))
 	    return PJ_SUCCESS;
 
@@ -1849,7 +1852,7 @@ static pj_status_t cmd_redirect_call(pj_cli_cmd_val *cval)
 	enum {
 	    ACCEPT_REPLACE, ACCEPT, REJECT, STOP
 	};
-	int choice = pj_strtol(&cval->argv[1]);
+	int choice = (int)pj_strtol(&cval->argv[1]);
 
 	switch (choice) {
 	case ACCEPT_REPLACE:
@@ -2549,7 +2552,7 @@ static pj_status_t cmd_sleep_handler(pj_cli_cmd_val *cval)
 {
     int delay;
 
-    delay = pj_strtoul(&cval->argv[1]);
+    delay = (int)pj_strtoul(&cval->argv[1]);
     if (delay < 0) delay = 0;
     pj_thread_sleep(delay);
 
