@@ -1,4 +1,4 @@
-/* $Id: call.hpp 5185 2015-10-02 02:08:17Z nanang $ */
+/* $Id: call.hpp 5417 2016-08-12 03:47:26Z ming $ */
 /*
  * Copyright (C) 2012-2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -951,6 +951,54 @@ struct OnCreateMediaTransportParam
     unsigned        flags;
 };
 
+/**
+ * SRTP crypto.
+ */
+struct SrtpCrypto
+{
+    /**
+     * Optional key. If empty, a random key will be autogenerated.
+     */
+    string	key;
+
+    /**
+     * Crypto name.
+     */
+    string	name;
+
+    /**
+     * Flags, bitmask from #pjmedia_srtp_crypto_option
+     */
+    unsigned	flags;
+};
+
+/**
+ * This structure contains parameters for Call::onCreateMediaTransportSrtp()
+ * callback.
+ */
+struct OnCreateMediaTransportSrtpParam
+{
+    /**
+     * The media index in the SDP for which the SRTP media transport
+     * will be used.
+     */
+    unsigned        		mediaIdx;
+    
+    /**
+     * Specify whether secure media transport should be used. Application
+     * can modify this only for initial INVITE.
+     * Valid values are PJMEDIA_SRTP_DISABLED, PJMEDIA_SRTP_OPTIONAL, and
+     * PJMEDIA_SRTP_MANDATORY.
+     */
+    pjmedia_srtp_use		srtpUse;
+    
+    /**
+     * Application can modify this to specify the cryptos and keys
+     * which are going to be used.
+     */
+    vector<SrtpCrypto>		cryptos;
+};
+
 /**
  * @}  // PJSUA2_Call_Data_Structure
  */
@@ -1743,6 +1791,20 @@ public:
     onCreateMediaTransport(OnCreateMediaTransportParam &prm)
     { PJ_UNUSED_ARG(prm); }
 
+    /**
+     * This callback is called when SRTP media transport is created.
+     * Application can modify the SRTP setting \a srtpOpt to specify
+     * the cryptos and keys which are going to be used. Note that
+     * application should not modify the field
+     * \a pjmedia_srtp_setting.close_member_tp and can only modify
+     * the field \a pjmedia_srtp_setting.use for initial INVITE.
+     *
+     * @param prm	Callback parameter.
+     */
+    virtual void
+    onCreateMediaTransportSrtp(OnCreateMediaTransportSrtpParam &prm)
+    { PJ_UNUSED_ARG(prm); }
+
 private:
     Account             &acc;
     pjsua_call_id 	 id;
