@@ -1,4 +1,4 @@
-/* $Id: endpoint.cpp 4887 2014-08-13 09:14:53Z nanang $ */
+/* $Id: endpoint.cpp 5522 2017-01-11 11:13:57Z ming $ */
 /* 
  * Copyright (C) 2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -44,7 +44,85 @@ Endpoint *Endpoint::instance_;
 
 ///////////////////////////////////////////////////////////////////////////////
 
+TlsInfo::TlsInfo()
+	: empty(true)
+{
+}
+
+bool TlsInfo::isEmpty() const
+{
+    return empty;
+}
+
+void TlsInfo::fromPj(const pjsip_tls_state_info &info)
+{
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
+    pj_ssl_sock_info *ssock_info = info.ssl_sock_info;
+    char straddr[PJ_INET6_ADDRSTRLEN+10];
+    const char *verif_msgs[32];
+    unsigned verif_msg_cnt;
+    
+    empty	= false;
+    established = PJ2BOOL(ssock_info->established);
+    protocol 	= ssock_info->proto;
+    cipher 	= ssock_info->cipher;
+    cipherName	= pj_ssl_cipher_name(ssock_info->cipher);
+    pj_sockaddr_print(&ssock_info->local_addr, straddr, sizeof(straddr), 3);
+    localAddr 	= straddr;
+    pj_sockaddr_print(&ssock_info->remote_addr, straddr, sizeof(straddr),3);
+    remoteAddr 	= straddr;
+    verifyStatus = ssock_info->verify_status;
+    if (ssock_info->local_cert_info)
+        localCertInfo.fromPj(*ssock_info->local_cert_info);
+    if (ssock_info->remote_cert_info)
+        remoteCertInfo.fromPj(*ssock_info->remote_cert_info);
+    
+    /* Dump server TLS certificate verification result */
+    verif_msg_cnt = PJ_ARRAY_SIZE(verif_msgs);
+    pj_ssl_cert_get_verify_status_strings(ssock_info->verify_status,
+    				      	  verif_msgs, &verif_msg_cnt);
+    for (unsigned i = 0; i < verif_msg_cnt; ++i) {
+        verifyMsgs.push_back(verif_msgs[i]);
+    }
+#endif
+}
+
+SslCertInfo::SslCertInfo()
+	: empty(true)
+{
+}
+
+bool SslCertInfo::isEmpty() const
+{
+    return empty;
+}
+
+void SslCertInfo::fromPj(const pj_ssl_cert_info &info)
+{
+    empty 	= false;
+    version 	= info.version;
+    pj_memcpy(serialNo, info.serial_no, sizeof(info.serial_no));
+    subjectCn 	= pj2Str(info.subject.cn);
+    subjectInfo = pj2Str(info.subject.info);
+    issuerCn 	= pj2Str(info.issuer.cn);
+    issuerInfo 	= pj2Str(info.issuer.info);
+    validityStart.fromPj(info.validity.start);
+    validityEnd.fromPj(info.validity.end);
+    validityGmt = PJ2BOOL(info.validity.gmt);
+    raw 	= pj2Str(info.raw);
+
+    for (unsigned i = 0; i < info.subj_alt_name.cnt; i++) {
+    	SslCertName cname;
+    	cname.type = info.subj_alt_name.entry[i].type;
+    	cname.name = pj2Str(info.subj_alt_name.entry[i].name);
+    	subjectAltName.push_back(cname);
+    }
+}
+
+///////////////////////////////////////////////////////////////////////////////
+
 UaConfig::UaConfig()
+: mainThreadOnly(false)
 {
     pjsua_config ua_cfg;
 
@@ -387,7 +465,8 @@ Endpoint::~Endpoint()
 	delete cur_media; /* this will remove itself from the list */
     }
 
-    clearCodecInfoList();
+    clearCodecInfoList(codecInfoList);
+    clearCodecInfoList(videoCodecInfoList);
 
     try {
 	libDestroy();
@@ -488,7 +567,7 @@ void Endpoint::logFunc(int level, const char *data, int len)
     LogEntry entry;
     entry.level = level;
     entry.msg = string(data, len);
-    entry.threadId = (long)pj_thread_this();
+    entry.threadId = (long)(size_t)pj_thread_this();
     entry.threadName = string(pj_thread_get_name(pj_thread_this()));
 
     ep.utilLogWrite(entry);
@@ -556,9 +635,20 @@ void Endpoint::on_transport_state( pjsip_transport *tp,
     OnTransportStateParam prm;
 
     prm.hnd = (TransportHandle)tp;
+    prm.type = tp->type_name;
     prm.state = state;
     prm.lastError = info ? info->status : PJ_SUCCESS;
 
+#if defined(PJSIP_HAS_TLS_TRANSPORT) && PJSIP_HAS_TLS_TRANSPORT!=0
+    if (!pj_ansi_stricmp(tp->type_name, "tls") && info->ext_info &&
+	(state == PJSIP_TP_STATE_CONNECTED || 
+	 ((pjsip_tls_state_info*)info->ext_info)->
+			         ssl_sock_info->verify_status != PJ_SUCCESS))
+    {
+    	prm.tlsInfo.fromPj(*((pjsip_tls_state_info*)info->ext_info));
+    }
+#endif
+
     ep.onTransportState(prm);
 }
 
@@ -600,6 +690,15 @@ void Endpoint::on_incoming_call(pjsua_acc_id acc_id, pjsua_call_id call_id,
 	return;
     }
 
+    pjsua_call *call = &pjsua_var.calls[call_id];
+    if (!call->incoming_data) {
+	/* This happens when the incoming call callback has been called from 
+	 * inside the on_create_media_transport() callback. So we simply 
+	 * return here to avoid calling	the callback twice. 
+	 */
+	return;
+    }
+
     /* call callback */
     OnIncomingCallParam prm;
     prm.callId = call_id;
@@ -607,6 +706,10 @@ void Endpoint::on_incoming_call(pjsua_acc_id acc_id, pjsua_call_id call_id,
 
     acc->onIncomingCall(prm);
 
+    /* Free cloned rdata. */
+    pjsip_rx_data_free_cloned(call->incoming_data);
+    call->incoming_data = NULL;
+
     /* disconnect if callback doesn't handle the call */
     pjsua_call_info ci;
 
@@ -814,6 +917,20 @@ void Endpoint::on_mwi_info(pjsua_acc_id acc_id,
     acc->onMwiInfo(prm);
 }
 
+void Endpoint::on_acc_find_for_incoming(const pjsip_rx_data *rdata,
+				        pjsua_acc_id* acc_id)
+{
+    OnSelectAccountParam prm;
+
+    pj_assert(rdata && acc_id);
+    prm.rdata.fromPj(*((pjsip_rx_data *)rdata));
+    prm.accountIndex = *acc_id;
+    
+    instance_->onSelectAccount(prm);
+    
+    *acc_id = prm.accountIndex;
+}
+
 void Endpoint::on_buddy_state(pjsua_buddy_id buddy_id)
 {
     Buddy *buddy = (Buddy*)pjsua_buddy_get_user_data(buddy_id);
@@ -892,8 +1009,15 @@ void Endpoint::on_call_sdp_created(pjsua_call_id call_id,
     
     /* Check if application modifies the SDP */
     if (orig_sdp != prm.sdp.wholeSdp) {
-        pjmedia_sdp_parse(pool, (char*)prm.sdp.wholeSdp.c_str(),
-                          prm.sdp.wholeSdp.size(), &sdp);
+        pjmedia_sdp_session *new_sdp;
+        pj_str_t dup_new_sdp;
+        pj_str_t new_sdp_str = {(char*)prm.sdp.wholeSdp.c_str(),
+        			(pj_ssize_t)prm.sdp.wholeSdp.size()};
+
+        pj_strdup(pool, &dup_new_sdp, &new_sdp_str);        
+        pjmedia_sdp_parse(pool, dup_new_sdp.ptr,
+                          dup_new_sdp.slen, &new_sdp);
+        pj_memcpy(sdp, new_sdp, sizeof(*sdp));
     }
 }
 
@@ -1072,6 +1196,25 @@ void Endpoint::on_call_rx_offer(pjsua_call_id call_id,
     *opt = prm.opt.toPj();
 }
 
+void Endpoint::on_call_tx_offer(pjsua_call_id call_id,
+				void *reserved,
+				pjsua_call_setting *opt)
+{
+    PJ_UNUSED_ARG(reserved);
+
+    Call *call = Call::lookup(call_id);
+    if (!call) {
+	return;
+    }
+
+    OnCallTxOfferParam prm;
+    prm.opt.fromPj(*opt);
+
+    call->onCallTxOffer(prm);
+
+    *opt = prm.opt.toPj();
+}
+
 pjsip_redirect_op Endpoint::on_call_redirected(pjsua_call_id call_id,
                                                const pjsip_uri *target,
                                                const pjsip_event *e)
@@ -1183,7 +1326,22 @@ Endpoint::on_create_media_transport(pjsua_call_id call_id,
 {
     Call *call = Call::lookup(call_id);
     if (!call) {
-	return base_tp;
+	pjsua_call *in_call = &pjsua_var.calls[call_id];
+	if (in_call->incoming_data) {
+	    /* This can happen when there is an incoming call but the
+	     * on_incoming_call() callback hasn't been called. So we need to 
+	     * call the callback here.
+	     */
+	    on_incoming_call(in_call->acc_id, call_id, in_call->incoming_data);
+
+	    /* New call should already be created by app. */
+	    call = Call::lookup(call_id);
+	    if (!call) {
+		return base_tp;
+	    }
+	} else {
+	    return base_tp;
+	}
     }
     
     OnCreateMediaTransportParam prm;
@@ -1196,6 +1354,53 @@ Endpoint::on_create_media_transport(pjsua_call_id call_id,
     return (pjmedia_transport *)prm.mediaTp;
 }
 
+void Endpoint::on_create_media_transport_srtp(pjsua_call_id call_id,
+                                    	      unsigned media_idx,
+                                    	      pjmedia_srtp_setting *srtp_opt)
+{
+    Call *call = Call::lookup(call_id);
+    if (!call) {
+	pjsua_call *in_call = &pjsua_var.calls[call_id];
+	if (in_call->incoming_data) {
+	    /* This can happen when there is an incoming call but the
+	     * on_incoming_call() callback hasn't been called. So we need to 
+	     * call the callback here.
+	     */
+	    on_incoming_call(in_call->acc_id, call_id, in_call->incoming_data);
+
+	    /* New call should already be created by app. */
+	    call = Call::lookup(call_id);
+	    if (!call) {
+		return;
+	    }
+	} else {
+	    return;
+	}
+    }
+    
+    OnCreateMediaTransportSrtpParam prm;
+    prm.mediaIdx = media_idx;
+    prm.srtpUse  = srtp_opt->use;
+    for (unsigned i = 0; i < srtp_opt->crypto_count; i++) {
+    	SrtpCrypto crypto;
+    	
+    	crypto.key   = pj2Str(srtp_opt->crypto[i].key);
+    	crypto.name  = pj2Str(srtp_opt->crypto[i].name);
+    	crypto.flags = srtp_opt->crypto[i].flags;
+    	prm.cryptos.push_back(crypto);
+    }
+    
+    call->onCreateMediaTransportSrtp(prm);
+    
+    srtp_opt->use = prm.srtpUse;
+    srtp_opt->crypto_count = prm.cryptos.size();
+    for (unsigned i = 0; i < srtp_opt->crypto_count; i++) {
+    	srtp_opt->crypto[i].key   = str2Pj(prm.cryptos[i].key);
+    	srtp_opt->crypto[i].name  = str2Pj(prm.cryptos[i].name);
+    	srtp_opt->crypto[i].flags = prm.cryptos[i].flags;
+    }
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /*
  * Endpoint library operations
@@ -1257,6 +1462,7 @@ void Endpoint::libInit(const EpConfig &prmEpConfig) throw(Error)
     ua_cfg.cb.on_typing2	= &Endpoint::on_typing2;
     ua_cfg.cb.on_mwi_info	= &Endpoint::on_mwi_info;
     ua_cfg.cb.on_buddy_state	= &Endpoint::on_buddy_state;
+    ua_cfg.cb.on_acc_find_for_incoming  = &Endpoint::on_acc_find_for_incoming;
 
     /* Call callbacks */
     ua_cfg.cb.on_call_state             = &Endpoint::on_call_state;
@@ -1271,11 +1477,14 @@ void Endpoint::libInit(const EpConfig &prmEpConfig) throw(Error)
     ua_cfg.cb.on_call_replace_request2  = &Endpoint::on_call_replace_request2;
     ua_cfg.cb.on_call_replaced          = &Endpoint::on_call_replaced;
     ua_cfg.cb.on_call_rx_offer          = &Endpoint::on_call_rx_offer;
+    ua_cfg.cb.on_call_tx_offer          = &Endpoint::on_call_tx_offer;
     ua_cfg.cb.on_call_redirected        = &Endpoint::on_call_redirected;
     ua_cfg.cb.on_call_media_transport_state =
         &Endpoint::on_call_media_transport_state;
     ua_cfg.cb.on_call_media_event       = &Endpoint::on_call_media_event;
     ua_cfg.cb.on_create_media_transport = &Endpoint::on_create_media_transport;
+    ua_cfg.cb.on_stun_resolution_complete = 
+    	&Endpoint::stun_resolve_cb;
 
     /* Init! */
     PJSUA2_CHECK_EXPR( pjsua_init(&ua_cfg, &log_cfg, &med_cfg) );
@@ -1310,6 +1519,12 @@ void Endpoint::libRegisterThread(const string &name) throw(Error)
     pj_status_t status;
 
     desc = (pj_thread_desc*)malloc(sizeof(pj_thread_desc));
+    if (!desc) {
+	PJSUA2_RAISE_ERROR(PJ_ENOMEM);
+    }
+
+    pj_bzero(desc, sizeof(pj_thread_desc));
+
     status = pj_thread_register(name.c_str(), *desc, &thread);
     if (status == PJ_SUCCESS) {
 	threadDescMap[thread] = desc;
@@ -1452,7 +1667,7 @@ void Endpoint::utilTimerCancel(Token prmTimerToken)
 IntVector Endpoint::utilSslGetAvailableCiphers() throw (Error)
 {
 #if PJ_HAS_SSL_SOCK
-    pj_ssl_cipher ciphers[64];
+    pj_ssl_cipher ciphers[PJ_SSL_SOCK_MAX_CIPHERS];
     unsigned count = PJ_ARRAY_SIZE(ciphers);
 
     PJSUA2_CHECK_EXPR( pj_ssl_cipher_get_availables(ciphers, &count) );
@@ -1481,6 +1696,21 @@ pj_stun_nat_type Endpoint::natGetType() throw(Error)
     return type;
 }
 
+void Endpoint::natUpdateStunServers(const StringVector &servers,
+				    bool wait) throw(Error)
+{
+    pj_str_t srv[MAX_STUN_SERVERS];
+    unsigned i, count = 0;
+
+    for (i=0; i<servers.size() && i<MAX_STUN_SERVERS; ++i) {
+	srv[count].ptr = (char*)servers[i].c_str();
+	srv[count].slen = servers[i].size();
+	++count;
+    }
+
+    PJSUA2_CHECK_EXPR(pjsua_update_stun_servers(count, srv, wait) );
+}
+
 void Endpoint::natCheckStunServers(const StringVector &servers,
 				   bool wait,
 				   Token token) throw(Error)
@@ -1552,6 +1782,11 @@ void Endpoint::transportClose(TransportId id) throw(Error)
     PJSUA2_CHECK_EXPR( pjsua_transport_close(id, PJ_FALSE) );
 }
 
+void Endpoint::transportShutdown(TransportHandle tp) throw(Error)
+{
+    PJSUA2_CHECK_EXPR( pjsip_transport_shutdown((pjsip_transport *)tp) );
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 /*
  * Call operations
@@ -1614,6 +1849,11 @@ AudDevManager &Endpoint::audDevManager()
     return audioDevMgr;
 }
 
+VidDevManager &Endpoint::vidDevManager()
+{
+    return videoDevMgr;
+}
+
 /*
  * Codec operations.
  */
@@ -1624,15 +1864,7 @@ const CodecInfoVector &Endpoint::codecEnum() throw(Error)
 
     PJSUA2_CHECK_EXPR( pjsua_enum_codecs(pj_codec, &count) );
 
-    pj_enter_critical_section();
-    clearCodecInfoList();
-    for (unsigned i=0; i<count; ++i) {
-	CodecInfo *codec_info = new CodecInfo;
-
-	codec_info->fromPj(pj_codec[i]);
-	codecInfoList.push_back(codec_info);
-    }
-    pj_leave_critical_section();
+    updateCodecInfoList(pj_codec, count, codecInfoList);
     return codecInfoList;
 }
 
@@ -1662,10 +1894,90 @@ void Endpoint::codecSetParam(const string &codec_id,
     PJSUA2_CHECK_EXPR( pjsua_codec_set_param(&codec_str, pj_param) );
 }
 
-void Endpoint::clearCodecInfoList()
+void Endpoint::clearCodecInfoList(CodecInfoVector &codec_list)
 {
-    for (unsigned i=0;i<codecInfoList.size();++i) {
-	delete codecInfoList[i];
+    for (unsigned i=0;i<codec_list.size();++i) {
+	delete codec_list[i];
     }
-    codecInfoList.clear();
+    codec_list.clear();
+}
+
+void Endpoint::updateCodecInfoList(pjsua_codec_info pj_codec[], unsigned count,
+				   CodecInfoVector &codec_list)
+{
+    pj_enter_critical_section();
+    clearCodecInfoList(codec_list);
+    for (unsigned i = 0; i<count; ++i) {
+	CodecInfo *codec_info = new CodecInfo;
+
+	codec_info->fromPj(pj_codec[i]);
+	codec_list.push_back(codec_info);
+    }
+    pj_leave_critical_section();
+}
+
+const CodecInfoVector &Endpoint::videoCodecEnum() throw(Error)
+{
+#if PJSUA_HAS_VIDEO
+    pjsua_codec_info pj_codec[MAX_CODEC_NUM];
+    unsigned count = MAX_CODEC_NUM;
+
+    PJSUA2_CHECK_EXPR(pjsua_vid_enum_codecs(pj_codec, &count));
+
+    updateCodecInfoList(pj_codec, count, videoCodecInfoList);
+#endif
+    return videoCodecInfoList;
+}
+
+void Endpoint::videoCodecSetPriority(const string &codec_id,
+				     pj_uint8_t priority) throw(Error)
+{
+#if PJSUA_HAS_VIDEO
+    pj_str_t codec_str = str2Pj(codec_id);
+    PJSUA2_CHECK_EXPR(pjsua_vid_codec_set_priority(&codec_str, priority));
+#else
+    PJ_UNUSED_ARG(codec_id);
+    PJ_UNUSED_ARG(priority);
+#endif
+}
+
+VidCodecParam Endpoint::getVideoCodecParam(const string &codec_id) const 
+								   throw(Error)
+{    
+    VidCodecParam codec_param;
+#if PJSUA_HAS_VIDEO
+    pjmedia_vid_codec_param pj_param;
+    pj_str_t codec_str = str2Pj(codec_id);    
+
+    PJSUA2_CHECK_EXPR(pjsua_vid_codec_get_param(&codec_str, &pj_param));
+    codec_param.fromPj(pj_param);
+#else
+    PJ_UNUSED_ARG(codec_id);
+#endif
+    return codec_param;
+}
+
+void Endpoint::setVideoCodecParam(const string &codec_id,
+				  const VidCodecParam &param) throw(Error)
+{
+#if PJSUA_HAS_VIDEO
+    pj_str_t codec_str = str2Pj(codec_id);
+    pjmedia_vid_codec_param pj_param = param.toPj();
+    
+    PJSUA2_CHECK_EXPR(pjsua_vid_codec_set_param(&codec_str, &pj_param));
+#else
+    PJ_UNUSED_ARG(codec_id);
+    PJ_UNUSED_ARG(param);
+#endif
+}
+
+void Endpoint::resetVideoCodecParam(const string &codec_id) throw(Error)
+{
+#if PJSUA_HAS_VIDEO
+    pj_str_t codec_str = str2Pj(codec_id);    
+    
+    PJSUA2_CHECK_EXPR(pjsua_vid_codec_set_param(&codec_str, NULL));
+#else
+    PJ_UNUSED_ARG(codec_id);    
+#endif	
 }
